#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "turtle.h"

#define LAMBDA (0.5)
#define N 512
#define SQR(x) ((x)*(x))

float density[N][N] ;

void reset_density()
{
  int i,j ;
  for (i=0; i<N; i++)
    for (j=0; j<N; j++) 
      density[i][j] = 0 ;
}

void add_density(int x0, int y0, int x1, int y1, int xc, int yc, int step)
{
  int i,j ;
  float vi,vj ;
  float ell = pow((double) 2, -step) ;
  float h = pow((double) ell, LAMBDA) ;
  if (random()%2) h=-h ;
  for (i=x0 ; i<=x1 ; i++) {
    vi = SQR(sin((double)(i-x0+1)/(x1-x0+2)*M_PI)) ;
    for (j=y0 ; j<=y1 ; j++) {
      vj = SQR(sin((float)(j-y0+1)/(y1-y0+2)*M_PI)) ;
      density[i][j] += vi*vj*h ;
    }
  }
}

void cloud(int x0, int y0, int x1, int y1, int step)
{
  int xc = (x1-x0)/2 + x0 ;
  int yc = (y1-y0)/2 + y0 ;
  if (step>8) return ;
  add_density(x0, y0, x1, y1, xc, yc, step) ;
  cloud(x0, y0, xc, yc, step+1) ;
  cloud(xc, y0, x1, yc, step+1) ;
  cloud(x0, yc, xc, y1, step+1) ;
  cloud(xc, yc, x1, y1, step+1) ;
}

main()
{
  int n ;
  int i,j ;
  float d,s,dmin=FLT_MAX,dmax=-FLT_MAX ;

  CON("localhost") ;
  CLR() ;

  n=16 ;
  while (n<=N) {
    srandom(1) ;
    reset_density() ;
    cloud(0, 0, n-1, n-1, 1) ;
    for (i=0 ; i<n; i++) {
      for (j=0; j<n; j++) {
	if (density[i][j]>dmax) dmax=density[i][j] ;
	if (density[i][j]<dmin) dmin=density[i][j] ;
      }
    }
    s = 512/n ;
    NORTH() ;
    LW(s) ;
    POINT() ;
    JUMP(-256,-256) ;
    PD() ;
    for (i=0 ; i<n; i++) {
      for (j=0; j<n; j++) {
	d = (density[i][j]-dmin)/(dmax-dmin) ;
	COL(d, d, 0.4+0.6*d) ;
	JUMP(i*s-256+s/2,j*s-256+s/2) ;
      }
    }
    PU() ;
    n=n*2 ;
  }
}

